(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('lodash'), require('azure-functions-ts-essentials'), require('typegoose')) :
	typeof define === 'function' && define.amd ? define(['exports', 'lodash', 'azure-functions-ts-essentials', 'typegoose'], factory) :
	(factory((global.azureFunctionsMongooser = global.azureFunctionsMongooser || {}),global._,global._azureFunctionsTsEssentials,global.typegoose));
}(this, (function (exports,_,azureFunctionsTsEssentials,typegoose) { 'use strict';

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var BaseDocument = (function (_super) {
    __extends(BaseDocument, _super);
    function BaseDocument() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return BaseDocument;
}(typegoose.Typegoose));

(function (ErrorType) {
    ErrorType["Missing"] = "missing";
    ErrorType["MissingField"] = "missing_field";
    ErrorType["Invalid"] = "invalid";
    ErrorType["AlreadyExists"] = "already_exists";
})(exports.ErrorType || (exports.ErrorType = {}));

var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
function connect(instance, connStr) {
    return new Promise(function (resolve, reject) {
        return instance.connect(connStr, { useMongoClient: true }, function (err) { return err
            ? reject(err)
            : resolve(); });
    });
}
function parseFields(rawFields) {
    if (!rawFields)
        return {};
    return rawFields.split(',')
        .map(function (cur) { return String(cur).trim(); })
        .reduce(function (acc, cur) {
        acc[cur] = 1;
        return acc;
    }, {});
}
function parsePopulation(rawPopulation) {
    if (!rawPopulation)
        return '';
    var obj = {};
    for (var _i = 0, _a = rawPopulation.split(','); _i < _a.length; _i++) {
        var item = _a[_i];
        obj = appendObject(obj, item);
    }
    return toPopulation(obj);
}
var appendObject = function (obj, path) {
    var keys = path.split(':');
    var lastKey = keys.pop();
    var lastObj = keys.reduce(function (acc, cur) { return acc[cur] = acc[cur] || {}; }, obj);
    lastObj[lastKey] = 'path';
    return obj;
};
var toPopulation = function (obj) {
    return _.transform(obj, function (res, value, key) {
        if (typeof (value) === 'object')
            res.push({
                path: key,
                populate: toPopulation(value)
            });
        else
            res.push({ path: key });
    }, []);
};
var getErrorResponse = function (err) {
    var status = azureFunctionsTsEssentials.HttpStatusCode.InternalServerError;
    var type = '';
    if (err.name === 'MongoError' && err.code === 11000) {
        status = azureFunctionsTsEssentials.HttpStatusCode.Conflict;
        type = exports.ErrorType.AlreadyExists;
    }
    else if (err.name === 'ValidationError') {
        status = azureFunctionsTsEssentials.HttpStatusCode.UnprocessableEntity;
        type = exports.ErrorType.MissingField;
    }
    return {
        status: status,
        body: {
            type: type,
            message: err.message
        }
    };
};
function clearCollection(instance, name) {
    var collection = instance.connection.collections[name];
    return collection.drop()
        .catch(function () { return clearCollection(instance, name); });
}
var Mongooser = (function () {
    function Mongooser(model) {
        this.model = model;
    }
    Mongooser.prototype.getOne = function (id, projection, population) {
        var query = this.model.findOne({ _id: id }, projection).populate(population).lean();
        return query.then(function (doc) {
            if (!doc)
                return Promise.resolve({
                    status: azureFunctionsTsEssentials.HttpStatusCode.NotFound
                });
            var data = doc;
            data._id = String(doc._id);
            return {
                status: azureFunctionsTsEssentials.HttpStatusCode.OK,
                body: __assign({ _id: data._id }, JSON.parse(JSON.stringify(data)))
            };
        })
            .catch(getErrorResponse);
    };
    Mongooser.prototype.getMany = function (projection, showInactive, population) {
        if (showInactive === void 0) { showInactive = false; }
        var query = this.model.find(!showInactive
            ? { isActive: true }
            : {}, projection).populate(population).lean();
        return query
            .then(function (docs) {
            var data = [];
            for (var _i = 0, _a = docs; _i < _a.length; _i++) {
                var item = _a[_i];
                item._id = String(item._id);
                data.push(__assign({ _id: item._id }, JSON.parse(JSON.stringify(item))));
            }
            return {
                status: azureFunctionsTsEssentials.HttpStatusCode.OK,
                body: {
                    data: data,
                    hasMore: false,
                    totalCount: data.length
                }
            };
        })
            .catch(getErrorResponse);
    };
    Mongooser.prototype.insertMany = function (req) {
        var contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: exports.ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: exports.ErrorType.Invalid
                }
            });
        return this.model.insertMany(req.body)
            .then(function (docs) {
            var data = [];
            for (var _i = 0, _a = docs; _i < _a.length; _i++) {
                var item = _a[_i];
                item._id = String(item._id);
                data.push(__assign({ _id: item._id }, JSON.parse(JSON.stringify(item))));
            }
            return {
                status: azureFunctionsTsEssentials.HttpStatusCode.Created,
                body: {
                    data: data,
                    hasMore: false,
                    totalCount: data.length
                }
            };
        })
            .catch(getErrorResponse);
    };
    Mongooser.prototype.updateOne = function (req, id) {
        var contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: exports.ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: exports.ErrorType.Invalid
                }
            });
        if (!id)
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: exports.ErrorType.Invalid
                }
            });
        return this.model.findOneAndUpdate({ _id: id }, req.body, { new: true }).lean()
            .then(function (doc) {
            if (!doc)
                return {
                    status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                    body: {
                        type: exports.ErrorType.Missing
                    }
                };
            else {
                var data = doc;
                data._id = String(doc._id);
                return {
                    status: azureFunctionsTsEssentials.HttpStatusCode.OK,
                    body: __assign({ _id: data._id }, JSON.parse(JSON.stringify(data)))
                };
            }
        })
            .catch(getErrorResponse);
    };
    Mongooser.prototype.deactivateOne = function (id) {
        if (!id)
            return Promise.resolve({
                status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest,
                body: {
                    type: exports.ErrorType.Missing
                }
            });
        return this.model.findOneAndUpdate({ _id: id }, { isActive: false }).lean()
            .then(function (doc) {
            if (!doc)
                return {
                    status: azureFunctionsTsEssentials.HttpStatusCode.BadRequest
                };
            else
                return {
                    status: azureFunctionsTsEssentials.HttpStatusCode.OK,
                    body: {
                        deactivated: true,
                        _id: String(doc._id)
                    }
                };
        })
            .catch(getErrorResponse);
    };
    return Mongooser;
}());

exports.BaseDocument = BaseDocument;
exports.connect = connect;
exports.parseFields = parseFields;
exports.parsePopulation = parsePopulation;
exports.clearCollection = clearCollection;
exports.Mongooser = Mongooser;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=azure-functions-mongooser.umd.js.map
